package flm.b4a.betterdialogs;

import java.util.Timer;
import java.util.TimerTask;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnKeyListener;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.text.Editable;
import android.text.Html;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.Msgbox;
import anywheresoftware.b4a.keywords.constants.DialogResponse;
import anywheresoftware.b4a.keywords.constants.KeyCodes;
import anywheresoftware.b4a.objects.drawable.BitmapDrawable;
import flm.b4a.maskededittext.MaskedEditTextWrapper;

@BA.ActivityObject
@BA.Version(2.11F)
@BA.ShortName("BetterDialogs")
@BA.Author("Frdric Leneuf-Magaud")
@BA.Events(values = {
		"Validation(Answer As String, CompactAnswer As String) As String",
		"BackKeyPressed As Boolean",
		"Closing(DlgResponse As Int) As Boolean"
})
@BA.DependsOn(values = { "MaskedEditText" })
public class BetterDialogs
{
	private static AlertDialog ad;
	private static Msgbox.DialogResponse dr;
	public static boolean CanceledOnTouchOutside = false;

	/**
	 * Closes the current dialog.
	 * @param Response One of the DialogResponse constants.
	 */
	public static void CloseDialog(int Response)
	{
		dr.res = Response;
		ad.cancel();
	}

	private static void AjouterIcone(AlertDialog.Builder b, Object Icon)
	{
		if (Icon != null) {
			if (Icon instanceof BitmapDrawable)
				b.setIcon(((BitmapDrawable)Icon).getObject());
			else if (Icon instanceof Drawable)
				b.setIcon((Drawable)Icon);
			else if (Icon instanceof Bitmap) {
				BitmapDrawable bd = new BitmapDrawable();
				bd.Initialize((Bitmap)Icon);
				b.setIcon(bd.getObject());
			}
		}
	}

	/**
	 *Shows a modal message box.
	 *Returns one of the DialogResponse values.<BR/>
	 * @param Title The dialog title. HTML tags (&lt;B&gt;, &lt;I&gt;, &lt;U&gt;, &lt;BR&gt;, &lt;Sup&gt;, &lt;Sub&gt;, &lt;Font Color=...&gt;, &lt;TT&gt;, &lt;H1&gt; to &lt;H6&gt;, etc.) are interpreted.
	 * @param Message The dialog message. HTML tags are interpreted.
	 * @param Positive The text to display for the "positive" button. Pass "" if you don't want to show the button.
	 * @param Cancel The text to display for the "cancel" button. Pass "" if you don't want to show the button.
	 * @param Negative The text to display for the "negative" button. Pass "" if you don't want to show the button.
	 * @param Icon An image (bitmap or drawable) that will be drawn near the title. Pass Null if you don't want to show an icon.
	 */
	public static int MsgBox(String Title, String Message, String Positive, String Cancel, String Negative, Object Icon, BA ba)
	{
		AlertDialog.Builder b = new AlertDialog.Builder(ba.context);
		b.setTitle(Html.fromHtml(Title)).setMessage(Html.fromHtml(Message));
		AjouterIcone(b, Icon);
		dr = new Msgbox.DialogResponse(false);
		if (Positive.length() > 0)
			b.setPositiveButton(Html.fromHtml(Positive), dr);
		if (Negative.length() > 0)
			b.setNegativeButton(Html.fromHtml(Negative), dr);
		if (Cancel.length() > 0)
			b.setNeutralButton(Html.fromHtml(Cancel), dr);
		ad = b.create();
		ad.setCancelable(true);
		ad.setCanceledOnTouchOutside(CanceledOnTouchOutside);
		Msgbox.msgbox(ad, false);
		return dr.res;
	}

	private static void FermerClavierVirtuel(BA ba)
	{
		// DoEvents
		Msgbox.sendCloseMyLoopMessage();
		Msgbox.waitForMessage(false, true);

		// Fermeture du clavier virtuel
		InputMethodManager imm = (InputMethodManager)BA.applicationContext.getSystemService(Context.INPUT_METHOD_SERVICE);
		imm.hideSoftInputFromWindow(ba.vg.getWindowToken(), 0);
	}

	@BA.ShortName("BD_InputBoxParams")
	public static class InputParams
	{
		public static final int INPUT_TYPE_NUMBERS = android.text.InputType.TYPE_CLASS_NUMBER;
		public static final int INPUT_TYPE_NUMBERS_WITH_SIGN = android.text.InputType.TYPE_CLASS_NUMBER|android.text.InputType.TYPE_NUMBER_FLAG_SIGNED;
		public static final int INPUT_TYPE_DECIMAL_NUMBERS = INPUT_TYPE_NUMBERS_WITH_SIGN|android.text.InputType.TYPE_NUMBER_FLAG_DECIMAL;
		public static final int INPUT_TYPE_PHONE = android.text.InputType.TYPE_CLASS_PHONE;
		public static final int INPUT_TYPE_TEXT = android.text.InputType.TYPE_CLASS_TEXT;
		public static final int INPUT_TYPE_TEXT_WITH_CAPS = android.text.InputType.TYPE_CLASS_TEXT|android.text.InputType.TYPE_TEXT_FLAG_CAP_SENTENCES;

		String Question = "";
		int QuestionTextSize = -1;
		int InputTextSize = -1;
		String Default = "";
		String Format = "";
		int CursorPosition = -1;
		public int Gravity = android.view.Gravity.TOP|android.view.Gravity.LEFT;
		String Hint = "";
		public int HintColor = Integer.MAX_VALUE;
		int InputType = INPUT_TYPE_TEXT;
		boolean Multiline = false;
		boolean PasswordMode = false;
		int SpaceBetween = 0;
		boolean WithSuggestions = true;
		String ValidationCallback = "";
		String Answer = "";
		String CompactAnswer = "";

		public void Initialize()
		{
			Question = "";
			QuestionTextSize = -1;
			InputTextSize = -1;
			Default = "";
			Format = "";
			CursorPosition = -1;
			Gravity = android.view.Gravity.TOP|android.view.Gravity.LEFT;
			Hint = "";
			HintColor = Integer.MAX_VALUE;
			InputType = INPUT_TYPE_TEXT;
			Multiline = false;
			PasswordMode = false;
			SpaceBetween = 0;
			ValidationCallback = "";
			WithSuggestions = true;
			Answer = "";
			CompactAnswer = "";
		}

		/**
		 * Gets or sets the initial text of the input field.
		 */
		public String getDefault()
		{
			return Default;
		}
		public void setDefault(String Text)
		{
			Default = Text;
		}

		/**
		 * Gets or sets the input mask.
		 * Mask characters:
		 * # = Digit
		 * L = Letter
		 * A = Alphanumeric
		 * H = Hexadecimal
		 * ? = Any character
		 * Examples:
		 * - Date &amp; time = "##/##/#### ##:##"
		 * - Phone number = "(###) ###-####"
		 * - IP address = "###.###.###.###"
		 * - MAC address = "HH:HH:HH:HH:HH:HH"
		 * - Parental lock password = "AAAA"
		 */
		public String getFormat()
		{
			return Format;
		}
		public void setFormat(String Mask)
		{
			Format = Mask;
		}

		/**
		 *Gets or sets the cursor position in the input field.
		 *It is set by default to the length of the default text.
		 */
		public int getCursorPosition()
		{
			return CursorPosition;
		}
		public void setCursorPosition(int Value)
		{
			CursorPosition = Math.max(0, Value);
		}

		/**
		 * Gets or sets the text that will appear when the input field is empty.
		 */
		public String getHint()
		{
			return Hint;
		}
		public void setHint(String Text)
		{
			Hint = Text;
		}

		/**
		 *Gets or sets the input type flag. This flag is used to determine the settings of the virtual keyboard.
		 *The value is one of the INPUT_TYPE constants.
		 */
		public int getInputType()
		{
			return InputType;
		}
		public void setInputType(int Type)
		{
			InputType = Type;
		}

		/**
		 * Gets or sets the text size of the input field.
		 */
		public int getInputTextSize()
		{
			return InputTextSize;
		}
		public void setInputTextSize(int Size)
		{
			InputTextSize = Size;
		}

		/**
		 * Gets or sets the text above the input field. HTML tags are interpreted.
		 */
		public String getQuestion()
		{
			return Question;
		}
		public void setQuestion(String Text)
		{
			Question = Text;
		}

		/**
		 * Gets or sets the text size of the question.
		 */
		public int getQuestionTextSize()
		{
			return QuestionTextSize;
		}
		public void setQuestionTextSize(int Size)
		{
			QuestionTextSize = Size;
		}

		/**
		 *Gets or sets whether the input field should be in multiline mode.
		 *Default = single line mode.
		 */
		public boolean getMultiline()
		{
			return Multiline;
		}
		public void setMultiline(boolean Mode)
		{
			Multiline = Mode;
		}

		/**
		 *Gets or sets whether the input field should be in password mode and hide the actual characters.
		 *Default = False
		 */
		public boolean getPasswordMode()
		{
			return PasswordMode;
		}
		public void setPasswordMode(boolean Enabled)
		{
			PasswordMode = Enabled;
		}

		/**
		 * Gets or sets the height of the gap between the question and the input field.
		 */
		public int getSpaceBetween()
		{
			return SpaceBetween;
		}
		public void setSpaceBetween(int Value)
		{
			SpaceBetween = Value;
		}

		/**
		 *Sets the function to call when the dialog is exited with the "positive" button.
		 *The callback function will return "" if the input is valid or an error message if the input is invalid.
		 *Example:<code>
		 *IP.ValidationCallback("Input_Validation")</code>
		 */
		public void setValidationCallback(String Callback)
		{
			ValidationCallback = Callback.toLowerCase();
		}

		/**
		 *Sets whether the virtual keyboard displays any dictionary-based candidates.
		 *This setting is ignored if Format is not empty or InputType is set to Numbers or Phone.
		 *Default = False
		 */
		public boolean getWithSuggestions()
		{
			return WithSuggestions;
		}
		public void setWithSuggestions(boolean Enabled)
		{
			WithSuggestions = Enabled;
		}

		/**
		 * Returns the user input.
		 */
		public String getAnswer()
		{
			return Answer;
		}

		/**
		 * Returns a copy of the user input without the leading and trailing spaces, and without the placeholders if Format is not empty.
		 */
		public String getCompactAnswer()
		{
			return CompactAnswer;
		}
	}

	/**
	 *Shows a modal input box.
	 *Returns one of the DialogResponse values.<BR/>
	 * @param Title The dialog title. HTML tags (&lt;B&gt;, &lt;I&gt;, &lt;U&gt;, &lt;BR&gt;, &lt;Sup&gt;, &lt;Sub&gt;, &lt;Font Color=...&gt;, &lt;TT&gt;, &lt;H1&gt; to &lt;H6&gt;, etc.) are interpreted.
	 * @param Params A set of parameters defining the behaviour and the appearance of the dialog.
	 * @param Positive The text to display for the "positive" button. Pass "" if you don't want to show the button.
	 * @param Cancel The text to display for the "cancel" button. Pass "" if you don't want to show the button.
	 * @param Negative The text to display for the "negative" button. Pass "" if you don't want to show the button.
	 * @param Icon An image (bitmap or drawable) that will be drawn near the title. Pass Null if you don't want to show an icon.
	 */
	public static int InputBox(String Title, final InputParams Params, String Positive, String Cancel, String Negative, Object Icon, BA ba)
	{
		AlertDialog.Builder b = new AlertDialog.Builder(ba.context);

		// Titre du dialogue
		b.setTitle(Html.fromHtml(Title));
		AjouterIcone(b, Icon);

		// Corps du dialogue
		LinearLayout llDlg = new LinearLayout(ba.context);
		llDlg.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
		llDlg.setOrientation(LinearLayout.VERTICAL);

		// Boutons
		dr = new Msgbox.DialogResponse(false);
		if (Positive.length() > 0)
			b.setPositiveButton(Html.fromHtml(Positive), dr);
		if (Negative.length() > 0)
			b.setNegativeButton(Html.fromHtml(Negative), dr);
		if (Cancel.length() > 0)
			b.setNeutralButton(Html.fromHtml(Cancel), dr);
		ad = b.create();

		// Calcul de 10dip
		float Densit = BA.applicationContext.getResources().getDisplayMetrics().density;
		int dip10 = (int)(10f * Densit);

		// Question
		if (Params.Question.length() > 0) {
			TextView text = new TextView(ba.context);
			LayoutParams tParams = new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
			tParams.setMargins(dip10, 0, dip10, Params.SpaceBetween);
			text.setLayoutParams(tParams);
			text.setGravity(android.view.Gravity.CENTER_VERTICAL|android.view.Gravity.LEFT);
			text.setText(Html.fromHtml(Params.Question));
			if (Params.QuestionTextSize > -1)
				text.setTextSize(Params.QuestionTextSize);
			llDlg.addView(text);
		}

		// Champ de saisie
		final MaskedEditTextWrapper edit = new MaskedEditTextWrapper();
		edit.Initialize(ba, "");
		LayoutParams eParams = new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT);
		eParams.setMargins(dip10, 0, dip10, 0);
		edit.getObject().setLayoutParams(eParams);
		edit.getObject().addTextChangedListener(new TextWatcher() {
			@Override
			public void afterTextChanged(Editable arg0) {}

			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {}

			@Override
			public void onTextChanged(CharSequence arg0, int arg1, int arg2, int arg3) { edit.ShowError(""); }
		});
		edit.setGravity(Params.Gravity);
		edit.setSingleLine(!Params.Multiline);
		if (Params.Multiline)
			edit.setWrap(true);
		if (Params.InputTextSize > -1)
			edit.setTextSize(Params.InputTextSize);
		edit.setPasswordMode(Params.PasswordMode);
		edit.setWithSuggestions(Params.WithSuggestions);
		edit.setInputType(Params.InputType);
		if (Params.Hint != "") {
			edit.setHint(Params.Hint);
			if (Params.HintColor != Integer.MAX_VALUE)
				edit.setHintColor(Params.HintColor);
			else
				Params.HintColor = edit.getHintColor();
		}
		edit.setFormat(Params.Format);
		edit.setText(Params.Default);
		if (Params.CursorPosition == -1)
			edit.setSelectionStart(Params.Default.length());
		else
			edit.setSelectionStart(Params.CursorPosition);
		llDlg.addView(edit.getObject());
		ad.setView(llDlg);

		// Paramtrage de l'annulation
		ad.setCancelable(true);
		ad.setCanceledOnTouchOutside(CanceledOnTouchOutside);

		// Ouverture du clavier virtuel
		ad.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE + WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN);

		// Affichage du dialogue et validation de la saisie
		boolean Valide = false;
		while (!Valide) {
			dr.res = DialogInterface.BUTTON_NEUTRAL;
			Msgbox.msgbox(ad, false);
			if (dr.res == DialogResponse.POSITIVE && ba.subExists(Params.ValidationCallback)) {
				String Msg = (String) ba.raiseEvent(Params, Params.ValidationCallback, new Object[] { edit.getText().toString(), edit.getCompactText() });
				Valide = (Msg.length() == 0);
				if (Msg.length() > 0)
					edit.ShowError(Msg);
			}
			else
				Valide = true;
		}

		// Fermeture du clavier virtuel
		FermerClavierVirtuel(ba);

		// Stockage de la saisie
		Params.Answer = edit.getText().toString();
		Params.CompactAnswer = edit.getCompactText();
		return dr.res;
	}

	private static void ClonerBouton(Button btnDlg, Button btnACopier)
	{
		if (btnDlg != null) {
			if (btnACopier.getBackground() != null)
				btnDlg.setBackgroundDrawable(btnACopier.getBackground());
			btnDlg.setEnabled(btnACopier.isEnabled());
			btnDlg.setGravity(btnACopier.getGravity());
			btnDlg.setTextColor(btnACopier.getTextColors());
			btnDlg.setTextSize(btnACopier.getTextSize());
			btnDlg.setTypeface(btnACopier.getTypeface());
		}
	}

	@BA.ShortName("BD_CustomDlgParams")
	public static class CustomDlgParams
	{
		public int DialogGravity;
		int MarginWidth, MarginHeight;
		Object Background;
		Object Title;
		int TitleWidth;
		int TitleHeight;
		Object Dialog;
		int DialogWidth;
		int DialogHeight;
		Object PositiveButton;
		Object CancelButton;
		Object NegativeButton;
		boolean OpenKeyboard;
		int DureAffichage;

		public void Initialize()
		{
			DialogGravity = android.view.Gravity.CENTER;
			float Densit = BA.applicationContext.getResources().getDisplayMetrics().density;
			MarginWidth = (int) (30f * Densit);
			MarginHeight = 0;
			Background = null;
			Title = null;
			TitleWidth = -1;
			TitleHeight = -1;
			Dialog = null;
			DialogWidth = -1;
			DialogHeight = -1;
			PositiveButton = null;
			CancelButton = null;
			NegativeButton = null;
			OpenKeyboard = false;
			DureAffichage = 0;
		}

		/**
		 * Gets or sets the dialog background. It can be a color or a drawable. Pass Null (default) if you want to keep the default background.
		 */
		public Object getBackground()
		{
			return Background;
		}
		public void setBackground(Object background)
		{
			Background = background;
		}

		/**
		 * Gets or sets the dialog title. It can be a view (a label, a panel, ...) or a string (the HTML tags in the string are interpreted).
		 */
		public Object getTitle()
		{
			return Title;
		}
		public void setTitle(Object title)
		{
			Title = title;
		}

		/**
		 * Gets or sets the width of the title. Relevant only when the title is a view.
		 */
		public int getTitleWidth()
		{
			return TitleWidth;
		}
		public void setTitleWidth(int Width)
		{
			TitleWidth = Width;
		}

		/**
		 * Gets or sets the height of the title. Relevant only when the title is a view.
		 */
		public int getTitleHeight()
		{
			return TitleHeight;
		}
		public void setTitleHeight(int Height)
		{
			TitleHeight = Height;
		}

		/**
		 * Gets or sets the dialog body. It can be any view.
		 */
		public Object getDialogBody()
		{
			return Dialog;
		}
		public void setDialogBody(Object dlgBody)
		{
			Dialog = dlgBody;
		}

		/**
		 * Gets or sets the width of the dialog body.
		 */
		public int getBodyWidth()
		{
			return DialogWidth;
		}
		public void setBodyWidth(int Width)
		{
			DialogWidth = Width;
		}

		/**
		 * Gets or sets the height of the dialog body.
		 */
		public int getBodyHeight()
		{
			return DialogHeight;
		}
		public void setBodyHeight(int Height)
		{
			DialogHeight = Height;
		}

		/**
		 * Gets or sets the height of the margin added above the dialog body. Should be set only when the gravity is set to TOP. Default = 0.
		 */
		public int getMarginHeight()
		{
			return MarginHeight;
		}
		public void setMarginHeight(int margin)
		{
			MarginHeight = margin;
		}

		/**
		 * Gets or sets the "positive" button to display. It can be a button or a string (the HTML tags in the string are interpreted). Pass Null (default) or "" if you don't want to show the button.
		 */
		public Object getPositiveButton()
		{
			return PositiveButton;
		}
		public void setPositiveButton(Object button)
		{
			PositiveButton = button;
		}

		/**
		 * Gets or sets the "cancel" button to display. It can be a button or a string (the HTML tags in the string are interpreted). Pass Null (default) or "" if you don't want to show the button.
		 */
		public Object getCancelButton()
		{
			return CancelButton;
		}
		public void setCancelButton(Object button)
		{
			CancelButton = button;
		}

		/**
		 * Gets or sets the "negative" button to display. It can be a button or a string (the HTML tags in the string are interpreted). Pass Null (default) or "" if you don't want to show the button.
		 */
		public Object getNegativeButton()
		{
			return NegativeButton;
		}
		public void setNegativeButton(Object button)
		{
			NegativeButton = button;
		}

		/**
		 * If True, open the virtual keyboard when the dialog is shown. Relevant only when the dialog contains an editable view. Default = False.
		 */
		public boolean getOpenKeyboard()
		{
			return OpenKeyboard;
		}
		public void setOpenKeyboard(boolean Enabled)
		{
			OpenKeyboard = Enabled;
		}

		/**
		 * Gets or sets the duration of the dialog, in seconds. Pass 0 (default) if you don't want to set a duration (the dialog will be dismissed only by an user action).
		 */
		public int getDuration()
		{
			return DureAffichage;
		}
		public void setDuration(int Dure)
		{
			DureAffichage = Dure;
		}
	}

	/**
	 *Shows a modal custom dialog.
	 *Returns one of the DialogResponse values.
	 *@param DlgParams A set of parameters defining the position and the contents of the dialog.
	 *@param EvtPrefix The prefix for the BackKeyPressed and Closing events.
	 */
	public static int CustomDialog(final CustomDlgParams DlgParams, final String EvtPrefix, final BA ba) throws Exception
	{
		ad = new AlertDialog.Builder(ba.context).create();

		// Emplacement du dialogue
		WindowManager.LayoutParams lp = ad.getWindow().getAttributes();
		lp.gravity = DlgParams.DialogGravity;
		ad.getWindow().setAttributes(lp);

		// Titre du dialogue
		if (DlgParams.Title != null) {
			if (DlgParams.Title instanceof String) {
				if (((String)DlgParams.Title).length() > 0)
					ad.setTitle(Html.fromHtml((String) DlgParams.Title));
			}
			else if (DlgParams.Title instanceof View) {
				if (DlgParams.TitleHeight < 0) {
					throw new Exception("Height < 0 is not allowed for the title");
				}
				LinearLayout flTitre = new LinearLayout(ba.context);
				if (DlgParams.Background instanceof Integer)
					flTitre.setBackgroundColor((Integer) DlgParams.Background);
				else if (DlgParams.Background instanceof Drawable)
					flTitre.setBackgroundDrawable((Drawable) DlgParams.Background);
				flTitre.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
				((View)DlgParams.Title).setLayoutParams(new LayoutParams(DlgParams.TitleWidth, DlgParams.TitleHeight));
				flTitre.addView((View)DlgParams.Title);
				ad.setCustomTitle(flTitre);
			}	
		}

		// Corps du dialogue
		if (DlgParams.DialogWidth <= 0) {
			throw new Exception("Width <= 0 is an invalid size for the dialog body");
		}
		else if (DlgParams.DialogHeight <= 0) {
			throw new Exception("Height <= 0 is an invalid size for the dialog body");
		}
		BALayout pnlBody;
		if (DlgParams.Dialog.getClass() == BALayout.class) {
			pnlBody = (BALayout) DlgParams.Dialog;
		}
		else {
			pnlBody = new BALayout(ba.context);
			View vwBody = (View) DlgParams.Dialog;
			pnlBody.addView(vwBody, new BALayout.LayoutParams(0, 0, LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
		}
		pnlBody.setLayoutParams(new LayoutParams(DlgParams.DialogWidth, DlgParams.DialogHeight));
		if (DlgParams.Background instanceof Integer)
			pnlBody.setBackgroundColor((Integer) DlgParams.Background);
		else if (DlgParams.Background instanceof Drawable)
			pnlBody.setBackgroundDrawable((Drawable) DlgParams.Background);
		ad.setView(pnlBody, 0, DlgParams.MarginHeight, 0, 0);

		// Boutons personnaliss
		dr = new Msgbox.DialogResponse(false);
		boolean BoutonPositif = false;
		boolean BoutonNeutre = false;
		boolean BoutonNegatif = false;
		if (DlgParams.PositiveButton != null) {
			if (DlgParams.PositiveButton instanceof String) {
				if (((String)DlgParams.PositiveButton).length() > 0)
					ad.setButton(DialogInterface.BUTTON_POSITIVE, Html.fromHtml((String)DlgParams.PositiveButton), dr);
			}
			else if (DlgParams.PositiveButton instanceof Button) {
				ad.setButton(DialogInterface.BUTTON_POSITIVE, Html.fromHtml((String)((Button)DlgParams.PositiveButton).getText()), dr);
				BoutonPositif = true;
			}
		}
		if (DlgParams.CancelButton != null) {
			if (DlgParams.CancelButton instanceof String) {
				if (((String)DlgParams.CancelButton).length() > 0)
					ad.setButton(DialogInterface.BUTTON_NEUTRAL, Html.fromHtml((String)DlgParams.CancelButton), dr);
			}
			else if (DlgParams.CancelButton instanceof Button) {
				ad.setButton(DialogInterface.BUTTON_NEUTRAL, Html.fromHtml((String)((Button)DlgParams.CancelButton).getText()), dr);
				BoutonNeutre = true;
			}
		}
		if (DlgParams.NegativeButton != null) {
			if (DlgParams.NegativeButton instanceof String) {
				if (((String)DlgParams.NegativeButton).length() > 0)
					ad.setButton(DialogInterface.BUTTON_NEGATIVE, Html.fromHtml((String)DlgParams.NegativeButton), dr);
			}
			else if (DlgParams.NegativeButton instanceof Button) {
				ad.setButton(DialogInterface.BUTTON_NEGATIVE, Html.fromHtml((String)((Button)DlgParams.NegativeButton).getText()), dr);
				BoutonNegatif = true;
			}
		}
		ad.show();
		if (BoutonPositif || BoutonNeutre || BoutonNegatif) {
			if (BoutonPositif)
				ClonerBouton(ad.getButton(DialogInterface.BUTTON_POSITIVE), (Button)DlgParams.PositiveButton);
			if (BoutonNeutre)
				ClonerBouton(ad.getButton(DialogInterface.BUTTON_NEUTRAL), (Button)DlgParams.CancelButton);
			if (BoutonNegatif)
				ClonerBouton(ad.getButton(DialogInterface.BUTTON_NEGATIVE), (Button)DlgParams.NegativeButton);
		}

		// Dimensionnement du dialogue
		FrameLayout frame = (FrameLayout) pnlBody.getParent();
		android.view.ViewGroup.LayoutParams fLP = frame.getLayoutParams();
		fLP.width = DlgParams.DialogWidth;
		fLP.height = DlgParams.DialogHeight + DlgParams.MarginHeight;
		frame.setLayoutParams(fLP);
		ad.getWindow().setLayout(DlgParams.DialogWidth + DlgParams.MarginWidth, LayoutParams.WRAP_CONTENT);

		// Paramtrage de l'annulation
		ad.setCancelable(true);
		ad.setCanceledOnTouchOutside(CanceledOnTouchOutside);

		// Prparation des vnements 
		String evtBackKey = null;
		String evtClosing = null;
		if (EvtPrefix.length() > 0) {
			evtBackKey = EvtPrefix.toLowerCase(BA.cul) + "_backkeypressed";
			evtClosing = EvtPrefix.toLowerCase(BA.cul) + "_closing";
			if (!ba.subExists(evtBackKey))
				evtBackKey = null;
			if (!ba.subExists(evtClosing))
				evtClosing = null;
		}

		// Interception de la touche Back
		if (evtBackKey != null) {
			final String evtName = evtBackKey;
			ad.setOnKeyListener(new OnKeyListener() {
				@Override
				public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event)
				{
					if (keyCode == KeyCodes.KEYCODE_BACK) {
						if (event.getAction() == KeyEvent.ACTION_UP) {
							if (!((Boolean) ba.raiseEvent(this, evtName, new Object[0])))
								ad.cancel();
						}
						return true;
					}
					else
						return false;
				}
			});
		}

		if (DlgParams.OpenKeyboard) {
			// Ouverture du clavier virtuel
			ad.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE + WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN);
		}

		Timer T = null;
		TimerTask TT = null;
		if (DlgParams.DureAffichage > 0) {
			// Mise en place d'une minuterie
			T = new Timer();
			TT = new TimerTask() {
				@Override
				public void run() {
					CloseDialog(DialogInterface.BUTTON_NEUTRAL);
				}
			};
			T.schedule(TT, DlgParams.DureAffichage * 1000);
		}

		// Affichage du dialogue
		boolean Fermer = false;
		while (!Fermer) {
			Msgbox.msgbox(ad, false);
			Fermer = true;

			// Dclenchement de l'vnement Closing
			if (evtClosing != null) {
				Object Result = ba.raiseEvent(null, evtClosing, new Object[] { dr.res }); 
				if (Result instanceof Boolean)
					Fermer = (boolean) Result;
			}
		}

		if (DlgParams.OpenKeyboard) {
			// Fermeture du clavier virtuel
			FermerClavierVirtuel(ba);
		}
		if (T != null) {
			// Arrt de la minuterie
			TT.cancel();
			T.cancel();
		}
		return dr.res;
	}
}
